/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.gpssimulator;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateSequence;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;
import nl.cloudfarming.client.fleet.model.gps.GPSTracker;

/**
 * This class simulates the functionality of a GPS tracker by simulating driving
 * around in circles.
 *
 * @author Wytse Visser
 */
public final class GPSSimulator extends GPSTracker {

    private double timeSinceEnabled = 0;
    
    @Override
    public Point getPosition() {
        final double totalCircleTime = 120;
        final double angularVelocity = 0.008 / (totalCircleTime * UPDATES_PER_SECOND);
        
        timeSinceEnabled += 1.0 / UPDATES_PER_SECOND;
        double longitudeDifference = (Math.sin(Math.PI * (timeSinceEnabled / totalCircleTime)) * angularVelocity);
        double longitude = getMachine().getLongitude() + longitudeDifference;
        
        double latitudeDifference = (Math.cos(Math.PI * (timeSinceEnabled / totalCircleTime)) * angularVelocity);
        double latitude = getMachine().getLatitude() + latitudeDifference;
        
        Coordinate[] coordinates = new Coordinate[]{new Coordinate(longitude, latitude)};
        CoordinateSequence coordSequence = new CoordinateArraySequence(coordinates);
        return new Point(coordSequence, new GeometryFactory());
    }
}
